"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const globby_1 = __importDefault(require("globby"));
const errors_1 = require("./errors");
function getRootPackage() {
    let rootPkg;
    try {
        rootPkg = require(path_1.join(process.cwd(), 'package.json'));
    }
    catch (err) {
        if (err.code === 'MODULE_NOT_FOUND') {
            throw new errors_1.WorkspaceReleaseWarning(`I couldn't found a package.json in your current working directory. I assume to be executed in the root of a workspace based yarn project.`);
        }
        else {
            throw err;
        }
    }
    if (!rootPkg.workspaces) {
        throw new errors_1.WorkspaceReleaseWarning(`Your package.json doesn't contain a "workspaces" property. I assume to be executed in the root of a workspace based yarn project.`);
    }
    if (!Array.isArray(rootPkg.workspaces)) {
        throw new errors_1.WorkspaceReleaseError(`The "workspaces" property in your package.json isn't an array.`);
    }
    if (!rootPkg.workspaces.length) {
        throw new errors_1.WorkspaceReleaseWarning(`The "workspaces" property in your package.json is empty.`);
    }
    return rootPkg;
}
exports.getRootPackage = getRootPackage;
class PackageInfo {
    constructor(file) {
        this.file = file;
        this.data = require(path_1.join(process.cwd(), file));
        this.dir = path_1.dirname(file);
        this.wasChanged = false;
    }
}
exports.PackageInfo = PackageInfo;
function getPackageInfos({ workspaces }) {
    return __awaiter(this, void 0, void 0, function* () {
        const files = yield globby_1.default(workspaces.map((pattern) => path_1.join(pattern, 'package.json')));
        const pkgInfos = files
            .map((file) => new PackageInfo(file))
            .sort(({ data: a }, { data: b }) => a.name < b.name ? -1 : a.name > b.name ? 1 : 0);
        return pkgInfos;
    });
}
exports.getPackageInfos = getPackageInfos;
//# sourceMappingURL=packages.js.map