"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
const chalk_1 = __importDefault(require("chalk"));
const inquirer_1 = require("inquirer");
const semver_1 = require("semver");
const fs_extra_1 = require("fs-extra");
const packages_1 = require("./packages");
const errors_1 = require("./errors");
const { gray, cyan } = chalk_1.default;
function processVersions() {
    return __awaiter(this, void 0, void 0, function* () {
        const rootPkg = packages_1.getRootPackage();
        const pkgInfos = (yield packages_1.getPackageInfos(rootPkg)).filter(({ data }) => !data.private);
        if (!pkgInfos.length) {
            const prettyWorkspaces = `["${rootPkg.workspaces.join('", "')}"]`;
            throw new errors_1.WorkspaceReleaseWarning(`I couldn't find non-private packages with the given workspace setting. ${gray(`(used config: ${prettyWorkspaces})`)}`);
        }
        pkgInfos.forEach(({ data: { name, version } }) => {
            if (!semver_1.valid(version)) {
                throw new errors_1.WorkspaceReleaseError(`The version "${version}" in your package "${name}" is not valid.`);
            }
        });
        for (const info of pkgInfos) {
            const { data: { name, version } } = info;
            const { shouldUpdate } = yield inquirer_1.prompt({
                name: 'shouldUpdate',
                message: `Do you want to update the version of ${cyan(name)}? ${gray(`(current version: ${version})`)}`,
                type: 'confirm',
                default: false
            });
            if (shouldUpdate) {
                const releaseTypes = [
                    'prerelease',
                    'prepatch',
                    'preminor',
                    'premajor',
                    'patch',
                    'minor',
                    'major'
                ];
                const { chosenVersion } = yield inquirer_1.prompt({
                    type: 'list',
                    name: 'chosenVersion',
                    message: 'What version update do you want to use?',
                    choices: releaseTypes.map((release) => {
                        const value = semver_1.inc(version, release);
                        return {
                            value,
                            name: `${release} ${gray(`(${value})`)}`
                        };
                    })
                });
                info.data.version = chosenVersion;
                info.wasChanged = true;
            }
        }
        const changedPkgs = pkgInfos.filter(({ wasChanged }) => wasChanged);
        if (!changedPkgs.length) {
            return console.log('No change was found. ✌');
        }
        yield Promise.all(changedPkgs.map(({ data, file }) => fs_extra_1.writeFile(file, JSON.stringify(data, null, 2))));
        const { shouldPush } = yield inquirer_1.prompt({
            name: 'shouldPush',
            message: `Should I commit and push all changes with tags?
${gray(`${changedPkgs
                .map(({ data: { name, version } }) => `  - ${name}@${version}\n`)
                .join('')}`)}`,
            type: 'confirm',
            default: false
        });
        if (!shouldPush) {
            return console.log(`I haven't committed or pushed the changes.`);
        }
        child_process_1.execSync('git add .');
        child_process_1.execSync('git commit -m "release"');
        changedPkgs.forEach(({ data: { name, version } }) => child_process_1.execSync(`git tag ${name}@${version}`));
        child_process_1.execSync('git push');
        child_process_1.execSync('git push --tags');
    });
}
exports.processVersions = processVersions;
//# sourceMappingURL=process-versions.js.map