"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
const semver_1 = require("semver");
const set_npm_auth_token_for_ci_1 = __importDefault(require("@hutson/set-npm-auth-token-for-ci"));
const packages_1 = require("./packages");
const errors_1 = require("./errors");
function getNameVersionPair(TRAVIS_TAG) {
    const isScopedPackage = TRAVIS_TAG[0] === '@';
    if (isScopedPackage) {
        const [name, version] = TRAVIS_TAG.substring(1).split('@');
        return ['@' + name, version];
    }
    else {
        return TRAVIS_TAG.split('@');
    }
}
function getTag(pkgInfo) {
    const isPrerelease = pkgInfo.data.version.includes('-');
    if (pkgInfo.data.publishConfig && pkgInfo.data.publishConfig.tag) {
        return `--tag ${pkgInfo.data.publishConfig.tag}`;
    }
    else if (isPrerelease) {
        return '--tag next';
    }
    else {
        return '--tag latest';
    }
}
function getAccess(pkgInfo) {
    if (pkgInfo.data.publishConfig && pkgInfo.data.publishConfig.access) {
        return `--access ${pkgInfo.data.publishConfig.access}`;
    }
    else {
        return '';
    }
}
function publish() {
    return __awaiter(this, void 0, void 0, function* () {
        const { TRAVIS_TAG } = process.env;
        if (!TRAVIS_TAG) {
            throw new errors_1.WorkspaceReleaseError(`The environment variable "TRAVIS_TAG" seems to be undefined.`);
        }
        const isScopedPackage = TRAVIS_TAG[0] === '@';
        if (isScopedPackage
            ? !TRAVIS_TAG.substring(1).includes('@')
            : !TRAVIS_TAG.includes('@')) {
            throw new errors_1.WorkspaceReleaseWarning(`This tag doesn't seem to be a tag which should be used for publishing packages, because it misses a @ to separate package name and version.`);
        }
        const [name, version] = getNameVersionPair(TRAVIS_TAG);
        if (!semver_1.valid(version)) {
            throw new errors_1.WorkspaceReleaseError(`The tagged version "${version}" for the package "${name}" is not a valid SemVer version.`);
        }
        const rootPkg = packages_1.getRootPackage();
        const pkgInfo = (yield packages_1.getPackageInfos(rootPkg)).find(({ data }) => data.name === name);
        if (!pkgInfo) {
            throw new errors_1.WorkspaceReleaseError(`I couldn't found a package with name "${name}" in your workspace.`);
        }
        if (pkgInfo.data.version !== version) {
            throw new errors_1.WorkspaceReleaseError(`The tagged version "${version}" and the version of "${pkgInfo.data.name}" in the package.json which is "${pkgInfo.data.version}" differ. This should not be the case, if workspace-release was used correctly.`);
        }
        // set auth token, which should be stored securely in process.env.NPM_TOKEN
        const currentCwd = process.cwd();
        process.chdir(pkgInfo.dir);
        set_npm_auth_token_for_ci_1.default();
        process.chdir(currentCwd);
        // triggers normal lifecycle hooks, but we don't need to change anything
        const options = { cwd: pkgInfo.dir, stdio: 'inherit' };
        child_process_1.execSync(`npm version --allow-same-version --no-git-tag-version ${version}`, options);
        const tag = getTag(pkgInfo);
        const access = getAccess(pkgInfo);
        child_process_1.execSync(`npm publish ${tag} ${access}`, options);
    });
}
exports.publish = publish;
//# sourceMappingURL=publish.js.map